# -*- coding: utf-8 -*-
"""
wagtail_mvc decorators
"""
from __future__ import unicode_literals


def wagtail_mvc_url(func):
    """
    Decorates an existing method responsible for generating a url
    prepends the parent url to the generated url to account for

    :param func: The method to decorate
    :return: Full url
    """
    def outer(self, *args, **kwargs):
        parts = self.get_parent().url.split('/')
        parts += func(self, *args, **kwargs).split('/')
        return '/{0}/'.format('/'.join([part for part in parts if part]))
    return outer



# -*- coding: utf-8 -*-
"""
Test app models for the wagtail_mvc app
"""
from __future__ import unicode_literals
from wagtail.wagtailcore.models import Page
from wagtail_mvc.models import WagtailMvcMixin


class TestModelOne(WagtailMvcMixin, Page):
    """
    Test model with no wagtail_url_conf attribute
    """


class TestModelTwo(WagtailMvcMixin, Page):
    """
    Test model with wagtail_url_conf attribute
    """
    wagtail_url_conf = 'wagtail_mvc.tests.test_app.urls'



# -*- coding: utf-8 -*-
"""
Test app views for the wagtail_mvc app
"""
from __future__ import unicode_literals
from django.template.response import TemplateResponse


def index(request, *args, **kwargs):
    """
    Index view

    :param request: Request instance
    :param args: Default positional args
    :param kwargs: Default keyword args

    :return: TemplateResponse instance
    """
    return TemplateResponse(request, 'test_app/index.html', kwargs)


def sub_page(request, *args, **kwargs):
    """
    Sub Page view

    :param request: Request instance
    :param args: Default positional args
    :param kwargs: Default keyword args

    :return: TemplateResponse instance
    """
    return TemplateResponse(request, 'test_app/sub_page.html', kwargs)



# -*- coding: utf-8 -*-
"""
Test app factories for the wagtail_mvc app
"""
from __future__ import unicode_literals

import factory

from wagtail_mvc.tests.test_app.models import TestModelOne, TestModelTwo


class TestModelOneFactory(factory.DjangoModelFactory):
    """
    Factory for generating TestModelOne instances
    """
    title = factory.Sequence(lambda x: 'test page one {0}'.format(x))
    depth = 0

    class Meta(object):
        """
        FactoryBoy properties
        """
        model = TestModelOne


class TestModelTwoFactory(factory.DjangoModelFactory):
    """
    Factory for generating TestModelTwo instances
    """
    title = factory.Sequence(lambda x: 'test page two {0}'.format(x))
    depth = 0

    class Meta(object):
        """
        FactoryBoy properties
        """
        model = TestModelTwo



# -*- coding: utf-8 -*-
"""
wagtail_mvc decorator tests
"""
from __future__ import unicode_literals
from django.test import TestCase
from mock import Mock
from wagtail_mvc.decorators import wagtail_mvc_url


class WagtailMvcUrlDecoratorTestCase(TestCase):
    """
    Tests the wagtail_mvc_url decorator
    """
    def setUp(self):
        """
        Sets up the test case
        """
        super(WagtailMvcUrlDecoratorTestCase, self).setUp()
        self.parent = Mock()
        self.parent.url = '/base/'
        self.instance = Mock()
        self.instance.get_parent = Mock(return_value=self.parent)

    def test_generates_correct_url(self):
        """
        The decorator should generate the correct url
        """
        func = wagtail_mvc_url(Mock(return_value='/foo/bar/'))
        self.assertEqual(func(self.instance), '/base/foo/bar/')

    def test_generates_correct_url_2(self):
        """
        The decorator should generate the correct url
        """
        func = wagtail_mvc_url(Mock(return_value='foo/bar'))
        self.assertEqual(func(self.instance), '/base/foo/bar/')

    def test_generates_correct_url_3(self):
        """
        The decorator should generate the correct url
        """
        self.parent.url = 'base'
        func = wagtail_mvc_url(Mock(return_value='foo/bar'))
        self.assertEqual(func(self.instance), '/base/foo/bar/')



# -*- coding: utf-8 -*-
"""
MVC model mixins for Wagtail CMS
"""
from __future__ import unicode_literals
from django.http import Http404
from django.core.urlresolvers import resolve
from wagtail.wagtailcore.models import Page, RouteResult


class WagtailMvcViewWrapper(object):
    """
    Simple wrapper around a view and corresponding page instance
    that allows the WagtailMvcMixin to return a routable view
    instead of a model with a serve method
    """
    def __init__(self, view, page):
        """
        Sets required attributes of the instance

        :param view: Callable view function
        :param page: Wagtail page model instance
        """
        self.serve = view
        self.page = page

    def get_view_restrictions(self):
        """
        Gets view restrictions from the page instance

        :return: Queryset of PageViewRestriction model instances
        """
        return self.page.get_view_restrictions()


class WagtailMvcMixin(object):
    """
    Model mixin that provides better separation
    between model and view logic for wagtail
    """
    def resolve_view(self, path_components):
        """
        Resolves the view to be rendered

        :param request:
        :param path_components:
        :return:
        """
        if path_components:
            path = '/{0}/'.format('/'.join(path_components))
        else:
            path = '/'
        specific = self.specific
        view, args, kwargs = resolve(path, urlconf=specific.wagtail_url_conf)
        kwargs['page'] = self
        view = WagtailMvcViewWrapper(view, self)
        return RouteResult(view, args, kwargs)

    def route(self, request, path_components):
        """
        Custom route method
        Uses wagtails default routing mechanism to get a route result
        then attempts to resolve a view object using the 'wagtail_url_conf'
        attribute on the routed page to get the associated view

        :param request: HttpRequest instance
        :param path_components: List of URL path components

        :return: Iterable containing ViewWrapper|Page, args, kwargs
        """
        if path_components:
            child_slug = path_components[0]
            remaining_components = path_components[1:]

            try:
                subpage = self.get_children().get(slug=child_slug)
            except Page.DoesNotExist:
                if hasattr(self, 'wagtail_url_conf'):
                    return self.resolve_view(path_components)
                else:
                    raise Http404
            else:
                return subpage.specific.route(request, remaining_components)

        else:
            # request is for this very page
            if self.live:
                if hasattr(self, 'wagtail_url_conf'):
                    return self.resolve_view(path_components)
                else:
                    return RouteResult(self)
            else:
                raise Http404



#!/usr/bin/env python
from __future__ import unicode_literals
from wagtail_mvc import __version__
from setuptools import setup, find_packages

setup(
      name='wagtail_mvc',
      version=__version__,
      description='Allows better separation between '
                  'models and views in Wagtail CMS',
      author='Dan Stringer',
      author_email='dan.stringer1983@googlemail.com',
      url='https://github.com/fatboystring/Wagtail-MVC/',
      download_url='https://github.com/fatboystring/Wagtail-MVC/tarball/0.1.0',
      packages=find_packages(exclude=['app']),
      license='MIT',
      classifiers=[
            'Development Status :: 4 - Beta',
            'Intended Audience :: Developers',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5'
      ],
      include_package_data=True,
      keywords=['wagtail', 'django', 'mvc']
)



# -*- coding: utf-8 -*-
"""
URLs for test_app.
"""
from __future__ import absolute_import, unicode_literals

from django.conf.urls import url

from wagtail_mvc.tests.test_app.views import index, sub_page

urlpatterns = [
    url(r'^sub-page/$', sub_page),
    url(r'^$', index),
]



# -*- coding: utf-8 -*-
"""
wagtail_mvc tests
"""
from __future__ import unicode_literals

from django.http import Http404
from django.test import RequestFactory, TestCase
from mock import Mock
from wagtail.wagtailcore.models import RouteResult

from wagtail_mvc.models import WagtailMvcViewWrapper
from wagtail_mvc.tests.test_app.factories import (
    TestModelOneFactory,
    TestModelTwoFactory
)


class WagtailMvcViewWrapperTestCase(TestCase):
    """
    Tests the WagtailMvcViewWrapper
    """
    def setUp(self):
        super(WagtailMvcViewWrapperTestCase, self).setUp()
        self.view = Mock()
        self.page = Mock(methods=['get_view_restrictions'])
        self.instance = WagtailMvcViewWrapper(self.view, self.page)

    def test_serve_calls_view(self):
        """
        The instances serve attribute should call the view
        """
        self.instance.serve()
        self.view.assert_called_with()

    def test_get_view_restrictions(self):
        """
        The method should call the get_view_restrictions method on the page
        """
        self.instance.get_view_restrictions()
        self.page.get_view_restrictions.assert_called_with()


class WagtailMvcMixinTestCase(TestCase):
    """
    Tests the WagtailMvcMixin
    """
    def setUp(self):
        super(WagtailMvcMixinTestCase, self).setUp()
        self.request = RequestFactory().get('/fake-path/')
        self.homepage = TestModelOneFactory.create(
            path="0002",
            depth=0,
            url_path="/",
            numchild=1
        )
        self.page_1 = TestModelTwoFactory.create(path="00020001", depth=1)

    def test_renders_if_wagtail_url_conf_not_defined(self):
        """
        The page should render if no wagtail_url_conf attribute is defined
        """
        response = self.homepage.route(self.request, [])
        self.assertIsInstance(response, RouteResult)
        self.assertEqual(response[0], self.homepage)

    def test_resolve_view_resolves_view(self):
        """
        The resolve_view method should return the correct data
        """
        response = self.page_1.route(self.request, [])
        self.assertIsInstance(response, RouteResult)
        self.assertIsInstance(response[0], WagtailMvcViewWrapper)
        self.assertTrue(callable(response[0].serve))
        self.assertEqual(response[0].page, self.page_1)
        self.assertEqual(response[1], [])
        self.assertEqual(response[2], {'page': self.page_1})

    def test_resolve_view_raises_404(self):
        """
        The resolve_view method should raise a Resolver404 exception
        """
        self.assertRaises(Http404, self.page_1.route, self.request, ['foo'])

    def test_url_config_used_to_serve_sub_page(self):
        """
        The url config should be used to serve a sub page
        """
        response = self.page_1.route(self.request, ['sub-page'])
        self.assertIsInstance(response, RouteResult)
        self.assertIsInstance(response[0], WagtailMvcViewWrapper)
        self.assertTrue(callable(response[0].serve))
        self.assertEqual(response[0].page, self.page_1)
        self.assertEqual(response[1], [])
        self.assertEqual(response[2], {'page': self.page_1})
